# This program contains the corresponding functions and code to reproduce
# the analyses from the paper A Bayesian Regression Model for Circular Data
# based on the projected Normal Distribution of Nuez-Antonio G.,
# Gutirrez-Pea E., and Escarela G.
#
# The program runs on R version 2.9.1
#
# Author(s): G. Nuez-Antonio and E. Gutirrez-Pea < eduardo@sigma.iimas.unam.mx >
# Maintainer: G. Nuez-Antonio < gab.nuneza@gmail.com >
# Date: July, 2009.

#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------
#-------------------------------------------------------------------------------

source(file="RegressionFunctions.R")

#-------------------------------------------------------------------------------
# Example 1: Data: Sample of size 100 from a projected normal distribution.

Example1<-read.table(file="Example1.dat")

E1<-CircReg(theta~1+x,theta~1+x+x2,data=Example1,tm=1000,t.lag=20)

B1<-E1$BI
B2<-E1$BII


if(0){

# Figure 1

nf<-layout(rbind(c(1,1,2,2),c(3,3,4,4),c(0,5,5,0)))
hist(B1[,1],freq=F,col="black", border="white", main="",xlab=expression(beta[0]^1:(-3.0)))
hist(B1[,2],freq=F,col="black", border="white", main="",xlab=expression(beta[1]^1:(1.0)))
hist(B2[,1],freq=F,col="black", border="white", main="",xlab=expression(beta[0]^2:(1.5)))
hist(B2[,2],freq=F,col="black", border="white", main="",xlab=expression(beta[1]^2:(-2.0)))
hist(B2[,3],freq=F,col="black", border="white", main="",xlab=expression(beta[2]^2:(3.0)))

}

#-------------------------------------------------------------------------------
# Example 2: Data: Sample of size 100 from a von Mises distribution.

Example2<-read.table(file="Example2.dat")

E2<-CircReg(theta~1,theta~1+x,data=Example2,tm=1000,t.lag=20)

B1<-E2$BI
B2<-E2$BII

if(0){

# Figure 2

X.1.aux<-model.matrix(theta~1,Example2)
p.1<-length(X.1.aux[1,])
X.1<-matrix(X.1.aux,ncol=p.1)

X.2.aux<-model.matrix(theta~1+x,Example2)
p.2<-length(X.2.aux[1,])
X.2<-matrix(X.2.aux,ncol=p.2)

nn<-10000
mu.star<-matrix(0,tm,2)
xy.star1<-rep(0,nn)
xy.star2<-rep(0,nn)
theta.star<-matrix(0,n,nn)
theta.star.q<-matrix(0,n,2)
medias.star<-rep(0,n)

n<-100    #sample size

for(j in 1:n)
  {
    mu.star[,1]<-c(B1%*%X.1[j,])
    mu.star[,2]<-c(B2%*%X.2[j,])
    xy.star1<-rnorm(nn,mean(mu.star[,1]),1)
    xy.star2<-rnorm(nn,mean(mu.star[,2]),1)
    theta.star[j,]<-atan2(xy.star2,xy.star1)%%(2*pi)
    medias.star[j]<-circ.stats(theta.star[j,],1)

  }

Mdat<-matrix(c(Example2$x,medias.star),n,2)
o<-order(Mdat[,1])
Mdat.o<-cbind(Mdat[o,1],Mdat[o,2])

par(mfrow=c(1,1))
plot(Example2$x,(Example2$theta)*180/pi,ylim=c(0,760),pch=19,xlab="x",ylab=expression(theta),main="",cex=1)
points(Example2$x,360+(Example2$theta)*180/pi,pch=19,cex=1)
points(Mdat.o[,1],Mdat.o[,2],pch="-",cex=1.1)
points(Mdat.o[,1],Mdat.o[,2]+360,pch="-",cex=1.1)

}

#-------------------------------------------------------------------------------
# Example 3: Periwinkles' Data

periwinkles<-read.table(file="Periwinkles.dat")

per<-CircReg(theta~1+x,data=periwinkles,tm=1000,t.lag=20,burn=5000,flag.lk=TRUE)

B1<-per$BI
B2<-per$BII


if(0){

# Figure 3

par(mfrow=c(2,2))
hist(B1[,1],freq=F,col="black",border="white",ylab="",main="",xlab=expression(beta[0]^1))
hist(B1[,2],freq=F,col="black",border="white",ylab="",main="",xlab=expression(beta[1]^1))
hist(B2[,1],freq=F,col="black",border="white",ylab="",main="",xlab=expression(beta[0]^2))
hist(B2[,2],freq=F,col="black",border="white",ylab="",main="",xlab=expression(beta[1]^2))

# Note: Figures 4 and 5 are obtained with the program MissingData.R

}

#-------------------------------------------------------------------------------
# Example 4: REDMET data.

red2m.aux<-read.csv(file="red2m.csv",header=T)
red2m.dat<-red2m.aux[,-1]

red2<-CircReg(theta~1+rh+tmp+wsp+hora.s+hora.c,data=red2m.dat,tm=1000,t.lag=20)

B1<-red2$BI
B2<-red2$BII

if(0){

# Table 2

library(TeachingDemos)
emp.hpd(B1[,1],conf=0.95)
emp.hpd(B1[,2],conf=0.95)
emp.hpd(B1[,3],conf=0.95)
emp.hpd(B1[,4],conf=0.95)
emp.hpd(B1[,5],conf=0.95)
emp.hpd(B1[,6],conf=0.95)
emp.hpd(B2[,1],conf=0.95)
emp.hpd(B2[,2],conf=0.95)
emp.hpd(B2[,3],conf=0.95)
emp.hpd(B2[,4],conf=0.95)
emp.hpd(B2[,5],conf=0.95)
emp.hpd(B2[,6],conf=0.95)

}



#-------------------------------------------------------------------------------



