# Program MissingData.R
# This program simulates a sample from posterior distribution of missing responses.
# See, Example 3 from "A Bayesian Regression Model for
# Circular Data based on the projected Normal Distribution" of Nuez-Antonio G.,
# Gutirrez-Pea E., and Escarela G.
#
# The program runs on R version 2.9.1
#
# Author(s): G. Nuez-Antonio and E. Gutirrez-Pea < eduardo@sigma.iimas.unam.mx >
# Maintainer: G. Nuez-Antonio < gab.nuneza@gmail.com >
# Date: July, 2009.
#
#-------------------------------------------------------------------------------
#--------------------------------------------------------------------------
#-------------------------------------------------------------------------------

#rm(list=ls())
#.libPaths("/usr/local/R/contrib")

library(MASS)
library(CircStats)

source(file="RegressionFunctions.R")

#-------------------------------------------------------------------------------

### Data

# Periwinkles data without theta_13=197 (x_13=5),
# theta_23=75 (x_23=57) and theta_1=67 (x_1=107).

x<-c(#107,
46,33,67,122,69,43,30,12,25,37,69,#5,
83,68,38,21,1,71,60,71,71,#57,
53,38,70,7,48,7,21,27,
5,57,107)
thetaa<-c(#67,
66,74,61,58,60,100,89,171,166,98,60,#197,
98,86,123,165,133,101,105,71,84,#75,
98,83,71,74,91,38,200,56,
-1000,-1000,-1000)
theta.barra<- 91.162

#-------------------------------------------------------------------------------

Id2<-diag(2)
Id3<-diag(3)
Id4<-diag(4)

index.miss<-which(thetaa==-1000)
thetaa[index.miss]<-theta.barra
n.miss<-length(index.miss)

theta<-(pi/180)*thetaa
n<-length(theta)

datose<-matrix(0,n,2)
datose<-cbind(cos(theta),sin(theta))


# Prior specification for "matrix" B.

m1<-c(0.0,0.0)
L1<-Id2*0.0001
m2<-c(0.0,0.0)
L2<-Id2*0.0001

# Design Matrix

  X<-matrix(c(rep(1,n),x),n,2)
# X<-matrix(c(rep(1,n),x,x^2),n,3)

# Posterior specification for "matrix" B.

  XtX<-t(X)%*%X
  Lstar1<-L1+XtX
  Lstar2<-L2+XtX
  Sigma1<-solve(Lstar1)
  Sigma2<-solve(Lstar2)
  L1m1<-L1%*%m1
  L2m2<-L2%*%m2

###----------------------  GIBBS SAMPLING   -----------------------------------

### Initial values

r<-rep(1,n)

Y<-r*datose

### Number of iterations

  kk<-20000

### Matrices for getting final sample (Number of iteraciones/ t.lag)

  t.lag<-1
  tm<-kk/t.lag
  B1<-matrix(0,tm,2)
  B2<-matrix(0,tm,2)
  ttheta.miss<-rep(0,n.miss)
  T.miss<-matrix(0,tm,n.miss)

#------------------ Burn-in period (10%*iterations)----------------------


for(k in 1:(kk/10))
{
   	
# Sampling of vectors beta1 and beta2.

XtY1<-t(X)%*%Y[,1]
XtY2<-t(X)%*%Y[,2]
mstar1<-c( Sigma1%*%(L1m1 + XtY1) )
mstar2<-c( Sigma2%*%(L2m2 + XtY2) )
beta1<-mvrnorm(1,mstar1,Sigma1)
beta2<-mvrnorm(1,mstar2,Sigma2)

# Sampling of missing data and vector r

for(j in 1:(n-n.miss) )
  {
    t.aux<-theta[j]
    mu.b1<-c(beta1%*%X[j,])
    mu.b2<-c(beta2%*%X[j,])
    b<-Dbd(t.aux,mu.b1,mu.b2)
    r[j]<-b+(  pnorm(b)/( dnorm(b)+b*pnorm(b) )   )
  }

for(jj in (n-n.miss+1):n )
  {  
    mu.b1<-c(beta1%*%X[jj,])
    mu.b2<-c(beta2%*%X[jj,])
    rt.miss<-rNPxy.1(c(mu.b1,mu.b2),Id2)
    r[jj]<-rt.miss[1]
    datose[jj,]<-c(cos(rt.miss[2]),sin(rt.miss[2]))
  }


Y<-r*datose
   
}
### ---------------------------- Iterations ------------------------------------

for(k in 1:kk)
{

flag1<-(k/1000)-trunc(k/1000)
if(flag1==0){print(k)}


# Sampling of vectors beta1 and beta2.

XtY1<-t(X)%*%Y[,1]
XtY2<-t(X)%*%Y[,2]
mstar1<-c( Sigma1%*%(L1m1 + XtY1) )
mstar2<-c( Sigma2%*%(L2m2 + XtY2) )
beta1<-mvrnorm(1,mstar1,Sigma1)
beta2<-mvrnorm(1,mstar2,Sigma2)

# Sampling of missing data and vector r

for(j in 1:(n-n.miss) )
  {
    t.aux<-theta[j]
    mu.b1<-c(beta1%*%X[j,])
    mu.b2<-c(beta2%*%X[j,])
    r[j]<-Metro(llrt,t.aux,mu.b1,mu.b2,1,10)
  }

ii<-k

for(jj in (n-n.miss+1):n )
  {    
    mu.b1<-c(beta1%*%X[jj,])
    mu.b2<-c(beta2%*%X[jj,])
    rt.miss<-rNPxy.1(c(mu.b1,mu.b2),Id2)
    ttheta.miss[jj]<-rt.miss[2]
    r[jj]<-rt.miss[1]
    datose[jj,]<-c(cos(rt.miss[2]),sin(rt.miss[2]))
    T.miss[ii,jj-(n-n.miss)]<-ttheta.miss[jj]
    rt.miss<-c(0.0,0.0)
  }            

Y<-r*datose


#---- Values of each iteration ------------------------------------------------

flag<-(k/t.lag)-trunc(k/t.lag)
if(flag==0)
{
ii<-k/t.lag

B1[ii,]<-beta1
B2[ii,]<-beta2
}
#-------------------------  Gibbs completed ------------------------------------

}

###------------------------------   Outs   -------------------------------------

if(0){

# Figure 4

nn<-10000
mu.star<-matrix(0,tm,2)
xy.star1<-rep(0,nn)
xy.star2<-rep(0,nn)
theta.star<-matrix(0,n,nn)
theta.star.q<-matrix(0,n,2)
medias.star<-rep(0,n)

for(j in 1:n)
  {
    mu.star[,1]<-c(B1%*%X[j,])
    mu.star[,2]<-c(B2%*%X[j,])
    xy.star1<-rnorm(nn,mean(mu.star[,1]),1)
    xy.star2<-rnorm(nn,mean(mu.star[,2]),1)
    theta.star[j,]<-atan2(xy.star2,xy.star1)%%(2*pi)
    medias.star[j]<-circ.stats(theta.star[j,],1)
    theta.star.q[j,]<-quantile(theta.star[j,],probs=c(0.05,0.975),names=F)
  }



plot(x,theta*180/pi,ylim=c(0,360),col="red",pch=19,ylab=expression(theta),main="Predictive Distribution")
points(x,medias.star,col="blue",pch=19)
for(j in 1:n)
{
lines(rep(x[j],2),theta.star.q[j,]*180/pi,col="red",lty="dotted")
}
bringToTop()

}
#-------------------------------------------------------------------------------

if(0){

# Figure 5

par(mfrow=c(3,2))
hist(T.miss[,1]*180/pi,col="black",border="white",xlab=expression(theta[13]: 197),main="",freq=F ,ylab="")
rose.diag(T.miss[,1],main="",bins=15,prop=2)
hist(T.miss[,2]*180/pi,col="black",border="white",xlab=expression(theta[23]: 75),main="" ,freq=F, ylab="")
rose.diag(T.miss[,2],main="",bins=20,prop=1.3)
hist(T.miss[,3]*180/pi,col="black",border="white",xlab=expression(theta[1]: 67),main="" ,freq=F, ylab="")
rose.diag(T.miss[,3],main="",bins=20,prop=1.3)
bringToTop()
}
#
#-------------------------------------------------------------------------------
#
# Last update: July, 2009, -gna.


